// Copyright (c) 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef ComputedStyleBase_h
#define ComputedStyleBase_h

#include "core/ComputedStyleBaseConstants.h"
#include "core/CoreExport.h"


namespace blink {

// The generated portion of ComputedStyle. For more info, see the header comment
// in ComputedStyle.h.
class CORE_EXPORT ComputedStyleBase {
 public:
  ALWAYS_INLINE ComputedStyleBase() :
      m_floating(static_cast<unsigned>(EFloat::None)),
      m_emptyCells(static_cast<unsigned>(EEmptyCells::Show)),
      m_visibility(static_cast<unsigned>(EVisibility::Visible))
  {}
  ~ComputedStyleBase() {}

  ALWAYS_INLINE ComputedStyleBase(const ComputedStyleBase& o) :
      m_floating(o.m_floating),
      m_emptyCells(o.m_emptyCells),
      m_visibility(o.m_visibility)
  {}

  bool operator==(const ComputedStyleBase& o) const {
    return true &&
        m_floating == o.m_floating &&
        m_emptyCells == o.m_emptyCells &&
        m_visibility == o.m_visibility &&
        true;
  }

  bool operator!=(const ComputedStyleBase& o) const { return !(*this == o); }

  inline bool inheritedEqual(const ComputedStyleBase& o) const {
    return true &&
        m_emptyCells == o.m_emptyCells &&
        m_visibility == o.m_visibility &&
        true;
  }

  inline bool independentInheritedEqual(const ComputedStyleBase& o) const {
    return true &&
        m_visibility == o.m_visibility &&
        true;
  }

  inline bool nonIndependentInheritedEqual(const ComputedStyleBase& o) const {
    return true &&
        m_emptyCells == o.m_emptyCells &&
        true;
  }

  inline bool nonInheritedEqual(const ComputedStyleBase& o) const {
    return true &&
        m_floating == o.m_floating &&
        true;
  }

  void setBitDefaults() {
    resetFloating();
    resetEmptyCells();
    resetVisibility();
  }

  enum IsAtShadowBoundary {
    AtShadowBoundary,
    NotAtShadowBoundary,
  };
  void inheritFrom(const ComputedStyleBase& inheritParent,
                   IsAtShadowBoundary isAtShadowBoundary = NotAtShadowBoundary);

  void copyNonInheritedFromCached(const ComputedStyleBase& other);

  // Fields.
  // TODO(sashab): Remove initialFoo() static methods and update callers to
  // use resetFoo(), which can be more efficient.
  // float
  inline static EFloat initialFloating() { return EFloat::None; }
  EFloat floating() const { return static_cast<EFloat>(m_floating); }
  void setFloating(EFloat v) { m_floating = static_cast<unsigned>(v); }
  inline void resetFloating() { m_floating = static_cast<unsigned>(EFloat::None); }

  // empty-cells
  inline static EEmptyCells initialEmptyCells() { return EEmptyCells::Show; }
  EEmptyCells emptyCells() const { return static_cast<EEmptyCells>(m_emptyCells); }
  void setEmptyCells(EEmptyCells v) { m_emptyCells = static_cast<unsigned>(v); }
  inline void resetEmptyCells() { m_emptyCells = static_cast<unsigned>(EEmptyCells::Show); }

  // visibility
  inline static EVisibility initialVisibility() { return EVisibility::Visible; }
  EVisibility visibility() const { return static_cast<EVisibility>(m_visibility); }
  void setVisibility(EVisibility v) { m_visibility = static_cast<unsigned>(v); }
  inline void resetVisibility() { m_visibility = static_cast<unsigned>(EVisibility::Visible); }

 protected:
  // Storage.
  unsigned m_floating : 2; // EFloat
  unsigned m_emptyCells : 1; // EEmptyCells
  unsigned m_visibility : 2; // EVisibility
};

} // namespace blink

#endif // ComputedStyleBase_h
