// This file is generated

// Copyright (c) 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#include "core/inspector/protocol/Security.h"

#include "core/inspector/protocol/Protocol.h"

namespace blink {
namespace protocol {
namespace Security {

// ------------- Enum values from types.

const char Metainfo::domainName[] = "Security";
const char Metainfo::commandPrefix[] = "Security.";
const char Metainfo::version[] = "1.2";

namespace SecurityStateEnum {
const char* Unknown = "unknown";
const char* Neutral = "neutral";
const char* Insecure = "insecure";
const char* Warning = "warning";
const char* Secure = "secure";
const char* Info = "info";
} // namespace SecurityStateEnum

std::unique_ptr<SecurityStateExplanation> SecurityStateExplanation::parse(protocol::Value* value, ErrorSupport* errors)
{
    if (!value || value->type() != protocol::Value::TypeObject) {
        errors->addError("object expected");
        return nullptr;
    }

    std::unique_ptr<SecurityStateExplanation> result(new SecurityStateExplanation());
    protocol::DictionaryValue* object = DictionaryValue::cast(value);
    errors->push();
    protocol::Value* securityStateValue = object->get("securityState");
    errors->setName("securityState");
    result->m_securityState = ValueConversions<String>::parse(securityStateValue, errors);
    protocol::Value* summaryValue = object->get("summary");
    errors->setName("summary");
    result->m_summary = ValueConversions<String>::parse(summaryValue, errors);
    protocol::Value* descriptionValue = object->get("description");
    errors->setName("description");
    result->m_description = ValueConversions<String>::parse(descriptionValue, errors);
    protocol::Value* hasCertificateValue = object->get("hasCertificate");
    errors->setName("hasCertificate");
    result->m_hasCertificate = ValueConversions<bool>::parse(hasCertificateValue, errors);
    errors->pop();
    if (errors->hasErrors())
        return nullptr;
    return result;
}

std::unique_ptr<protocol::DictionaryValue> SecurityStateExplanation::serialize() const
{
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    result->setValue("securityState", ValueConversions<String>::serialize(m_securityState));
    result->setValue("summary", ValueConversions<String>::serialize(m_summary));
    result->setValue("description", ValueConversions<String>::serialize(m_description));
    result->setValue("hasCertificate", ValueConversions<bool>::serialize(m_hasCertificate));
    return result;
}

std::unique_ptr<SecurityStateExplanation> SecurityStateExplanation::clone() const
{
    ErrorSupport errors;
    return parse(serialize().get(), &errors);
}

std::unique_ptr<InsecureContentStatus> InsecureContentStatus::parse(protocol::Value* value, ErrorSupport* errors)
{
    if (!value || value->type() != protocol::Value::TypeObject) {
        errors->addError("object expected");
        return nullptr;
    }

    std::unique_ptr<InsecureContentStatus> result(new InsecureContentStatus());
    protocol::DictionaryValue* object = DictionaryValue::cast(value);
    errors->push();
    protocol::Value* ranMixedContentValue = object->get("ranMixedContent");
    errors->setName("ranMixedContent");
    result->m_ranMixedContent = ValueConversions<bool>::parse(ranMixedContentValue, errors);
    protocol::Value* displayedMixedContentValue = object->get("displayedMixedContent");
    errors->setName("displayedMixedContent");
    result->m_displayedMixedContent = ValueConversions<bool>::parse(displayedMixedContentValue, errors);
    protocol::Value* ranContentWithCertErrorsValue = object->get("ranContentWithCertErrors");
    errors->setName("ranContentWithCertErrors");
    result->m_ranContentWithCertErrors = ValueConversions<bool>::parse(ranContentWithCertErrorsValue, errors);
    protocol::Value* displayedContentWithCertErrorsValue = object->get("displayedContentWithCertErrors");
    errors->setName("displayedContentWithCertErrors");
    result->m_displayedContentWithCertErrors = ValueConversions<bool>::parse(displayedContentWithCertErrorsValue, errors);
    protocol::Value* ranInsecureContentStyleValue = object->get("ranInsecureContentStyle");
    errors->setName("ranInsecureContentStyle");
    result->m_ranInsecureContentStyle = ValueConversions<String>::parse(ranInsecureContentStyleValue, errors);
    protocol::Value* displayedInsecureContentStyleValue = object->get("displayedInsecureContentStyle");
    errors->setName("displayedInsecureContentStyle");
    result->m_displayedInsecureContentStyle = ValueConversions<String>::parse(displayedInsecureContentStyleValue, errors);
    errors->pop();
    if (errors->hasErrors())
        return nullptr;
    return result;
}

std::unique_ptr<protocol::DictionaryValue> InsecureContentStatus::serialize() const
{
    std::unique_ptr<protocol::DictionaryValue> result = DictionaryValue::create();
    result->setValue("ranMixedContent", ValueConversions<bool>::serialize(m_ranMixedContent));
    result->setValue("displayedMixedContent", ValueConversions<bool>::serialize(m_displayedMixedContent));
    result->setValue("ranContentWithCertErrors", ValueConversions<bool>::serialize(m_ranContentWithCertErrors));
    result->setValue("displayedContentWithCertErrors", ValueConversions<bool>::serialize(m_displayedContentWithCertErrors));
    result->setValue("ranInsecureContentStyle", ValueConversions<String>::serialize(m_ranInsecureContentStyle));
    result->setValue("displayedInsecureContentStyle", ValueConversions<String>::serialize(m_displayedInsecureContentStyle));
    return result;
}

std::unique_ptr<InsecureContentStatus> InsecureContentStatus::clone() const
{
    ErrorSupport errors;
    return parse(serialize().get(), &errors);
}

// ------------- Enum values from params.


// ------------- Frontend notifications.

void Frontend::securityStateChanged(const String& securityState, Maybe<protocol::Array<protocol::Security::SecurityStateExplanation>> explanations, Maybe<protocol::Security::InsecureContentStatus> insecureContentStatus, Maybe<bool> schemeIsCryptographic)
{
    std::unique_ptr<protocol::DictionaryValue> jsonMessage = DictionaryValue::create();
    jsonMessage->setString("method", "Security.securityStateChanged");
    std::unique_ptr<protocol::DictionaryValue> paramsObject = DictionaryValue::create();
    paramsObject->setValue("securityState", ValueConversions<String>::serialize(securityState));
    if (explanations.isJust())
        paramsObject->setValue("explanations", ValueConversions<protocol::Array<protocol::Security::SecurityStateExplanation>>::serialize(explanations.fromJust()));
    if (insecureContentStatus.isJust())
        paramsObject->setValue("insecureContentStatus", ValueConversions<protocol::Security::InsecureContentStatus>::serialize(insecureContentStatus.fromJust()));
    if (schemeIsCryptographic.isJust())
        paramsObject->setValue("schemeIsCryptographic", ValueConversions<bool>::serialize(schemeIsCryptographic.fromJust()));
    jsonMessage->setObject("params", std::move(paramsObject));
    if (m_frontendChannel)
        m_frontendChannel->sendProtocolNotification(jsonMessage->toJSONString());
}

void Frontend::flush()
{
    m_frontendChannel->flushProtocolNotifications();
}

// --------------------- Dispatcher.

class DispatcherImpl : public protocol::DispatcherBase {
public:
    DispatcherImpl(FrontendChannel* frontendChannel, Backend* backend)
        : DispatcherBase(frontendChannel)
        , m_backend(backend) {
    }
    ~DispatcherImpl() override { }
    DispatchResponse::Status dispatch(int callId, const String& method, std::unique_ptr<protocol::DictionaryValue> messageObject) override;

protected:
    using CallHandler = DispatchResponse::Status (DispatcherImpl::*)(int callId, std::unique_ptr<DictionaryValue> messageObject, ErrorSupport* errors);
    using DispatchMap = protocol::HashMap<String, CallHandler>;
    DispatchMap m_dispatchMap;


    Backend* m_backend;
};

DispatchResponse::Status DispatcherImpl::dispatch(int callId, const String& method, std::unique_ptr<protocol::DictionaryValue> messageObject)
{
    protocol::HashMap<String, CallHandler>::iterator it = m_dispatchMap.find(method);
    if (it == m_dispatchMap.end()) {
        reportProtocolError(callId, DispatchResponse::kMethodNotFound, "'" + method + "' wasn't found", nullptr);
        return DispatchResponse::kError;
    }

    protocol::ErrorSupport errors;
    return (this->*(it->second))(callId, std::move(messageObject), &errors);
}


// static
void Dispatcher::wire(UberDispatcher* dispatcher, Backend* backend)
{
    dispatcher->registerBackend("Security", std::unique_ptr<protocol::DispatcherBase>(new DispatcherImpl(dispatcher->channel(), backend)));
}

} // Security
} // namespace blink
} // namespace protocol
