// This file is generated

// Copyright (c) 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef blink_protocol_Target_h
#define blink_protocol_Target_h

#include "core/CoreExport.h"
#include "core/inspector/protocol/Protocol.h"
// For each imported domain we generate a ValueConversions struct instead of a full domain definition
// and include Domain::API version from there.

namespace blink {
namespace protocol {
namespace Target {

// ------------- Forward and enum declarations.
// 
using TargetID = String;
// 
using BrowserContextID = String;
// 
class TargetInfo;
// 
class RemoteLocation;

// ------------- Type and builder declarations.

// 
class CORE_EXPORT TargetInfo {
    PROTOCOL_DISALLOW_COPY(TargetInfo);
public:
    static std::unique_ptr<TargetInfo> parse(protocol::Value* value, ErrorSupport* errors);

    ~TargetInfo() { }

    String getTargetId() { return m_targetId; }
    void setTargetId(const String& value) { m_targetId = value; }

    String getType() { return m_type; }
    void setType(const String& value) { m_type = value; }

    String getTitle() { return m_title; }
    void setTitle(const String& value) { m_title = value; }

    String getUrl() { return m_url; }
    void setUrl(const String& value) { m_url = value; }

    std::unique_ptr<protocol::DictionaryValue> serialize() const;
    std::unique_ptr<TargetInfo> clone() const;

    template<int STATE>
    class TargetInfoBuilder {
    public:
        enum {
            NoFieldsSet = 0,
          TargetIdSet = 1 << 1,
          TypeSet = 1 << 2,
          TitleSet = 1 << 3,
          UrlSet = 1 << 4,
            AllFieldsSet = (TargetIdSet | TypeSet | TitleSet | UrlSet | 0)};


        TargetInfoBuilder<STATE | TargetIdSet>& setTargetId(const String& value)
        {
            static_assert(!(STATE & TargetIdSet), "property targetId should not be set yet");
            m_result->setTargetId(value);
            return castState<TargetIdSet>();
        }

        TargetInfoBuilder<STATE | TypeSet>& setType(const String& value)
        {
            static_assert(!(STATE & TypeSet), "property type should not be set yet");
            m_result->setType(value);
            return castState<TypeSet>();
        }

        TargetInfoBuilder<STATE | TitleSet>& setTitle(const String& value)
        {
            static_assert(!(STATE & TitleSet), "property title should not be set yet");
            m_result->setTitle(value);
            return castState<TitleSet>();
        }

        TargetInfoBuilder<STATE | UrlSet>& setUrl(const String& value)
        {
            static_assert(!(STATE & UrlSet), "property url should not be set yet");
            m_result->setUrl(value);
            return castState<UrlSet>();
        }

        std::unique_ptr<TargetInfo> build()
        {
            static_assert(STATE == AllFieldsSet, "state should be AllFieldsSet");
            return std::move(m_result);
        }

    private:
        friend class TargetInfo;
        TargetInfoBuilder() : m_result(new TargetInfo()) { }

        template<int STEP> TargetInfoBuilder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<TargetInfoBuilder<STATE | STEP>*>(this);
        }

        std::unique_ptr<protocol::Target::TargetInfo> m_result;
    };

    static TargetInfoBuilder<0> create()
    {
        return TargetInfoBuilder<0>();
    }

private:
    TargetInfo()
    {
    }

    String m_targetId;
    String m_type;
    String m_title;
    String m_url;
};


// 
class CORE_EXPORT RemoteLocation {
    PROTOCOL_DISALLOW_COPY(RemoteLocation);
public:
    static std::unique_ptr<RemoteLocation> parse(protocol::Value* value, ErrorSupport* errors);

    ~RemoteLocation() { }

    String getHost() { return m_host; }
    void setHost(const String& value) { m_host = value; }

    int getPort() { return m_port; }
    void setPort(int value) { m_port = value; }

    std::unique_ptr<protocol::DictionaryValue> serialize() const;
    std::unique_ptr<RemoteLocation> clone() const;

    template<int STATE>
    class RemoteLocationBuilder {
    public:
        enum {
            NoFieldsSet = 0,
          HostSet = 1 << 1,
          PortSet = 1 << 2,
            AllFieldsSet = (HostSet | PortSet | 0)};


        RemoteLocationBuilder<STATE | HostSet>& setHost(const String& value)
        {
            static_assert(!(STATE & HostSet), "property host should not be set yet");
            m_result->setHost(value);
            return castState<HostSet>();
        }

        RemoteLocationBuilder<STATE | PortSet>& setPort(int value)
        {
            static_assert(!(STATE & PortSet), "property port should not be set yet");
            m_result->setPort(value);
            return castState<PortSet>();
        }

        std::unique_ptr<RemoteLocation> build()
        {
            static_assert(STATE == AllFieldsSet, "state should be AllFieldsSet");
            return std::move(m_result);
        }

    private:
        friend class RemoteLocation;
        RemoteLocationBuilder() : m_result(new RemoteLocation()) { }

        template<int STEP> RemoteLocationBuilder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<RemoteLocationBuilder<STATE | STEP>*>(this);
        }

        std::unique_ptr<protocol::Target::RemoteLocation> m_result;
    };

    static RemoteLocationBuilder<0> create()
    {
        return RemoteLocationBuilder<0>();
    }

private:
    RemoteLocation()
    {
          m_port = 0;
    }

    String m_host;
    int m_port;
};


// ------------- Backend interface.

class CORE_EXPORT Backend {
public:
    virtual ~Backend() { }

    virtual DispatchResponse setAutoAttach(bool in_autoAttach, bool in_waitForDebuggerOnStart) = 0;
    virtual DispatchResponse sendMessageToTarget(const String& in_targetId, const String& in_message) = 0;

    virtual DispatchResponse disable()
    {
        return DispatchResponse::OK();
    }
};

// ------------- Frontend interface.

class CORE_EXPORT Frontend {
public:
    explicit Frontend(FrontendChannel* frontendChannel) : m_frontendChannel(frontendChannel) { }
    void attachedToTarget(std::unique_ptr<protocol::Target::TargetInfo> targetInfo, bool waitingForDebugger);
    void detachedFromTarget(const String& targetId);
    void receivedMessageFromTarget(const String& targetId, const String& message);

    void flush();
private:
    FrontendChannel* m_frontendChannel;
};

// ------------- Dispatcher.

class CORE_EXPORT Dispatcher {
public:
    static void wire(UberDispatcher*, Backend*);

private:
    Dispatcher() { }
};

// ------------- Metainfo.

class CORE_EXPORT Metainfo {
public:
    using BackendClass = Backend;
    using FrontendClass = Frontend;
    using DispatcherClass = Dispatcher;
    static const char domainName[];
    static const char commandPrefix[];
    static const char version[];
};

} // namespace Target
} // namespace blink
} // namespace protocol

#endif // !defined(blink_protocol_Target_h)
