// This file is generated

// Copyright (c) 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef blink_protocol_Tracing_h
#define blink_protocol_Tracing_h

#include "core/CoreExport.h"
#include "core/inspector/protocol/Protocol.h"
// For each imported domain we generate a ValueConversions struct instead of a full domain definition
// and include Domain::API version from there.
#include "core/inspector/protocol/IO.h"

namespace blink {
namespace protocol {
namespace Tracing {

// ------------- Forward and enum declarations.
// Configuration for memory dump. Used only when "memory-infra" category is enabled.
using MemoryDumpConfig = Object;
// 
class TraceConfig;

namespace Start {
namespace TransferModeEnum {
CORE_EXPORT extern const char* ReportEvents;
CORE_EXPORT extern const char* ReturnAsStream;
} // TransferModeEnum
} // Start

// ------------- Type and builder declarations.

// 
class CORE_EXPORT TraceConfig {
    PROTOCOL_DISALLOW_COPY(TraceConfig);
public:
    static std::unique_ptr<TraceConfig> parse(protocol::Value* value, ErrorSupport* errors);

    ~TraceConfig() { }

    struct CORE_EXPORT RecordModeEnum {
        static const char* RecordUntilFull;
        static const char* RecordContinuously;
        static const char* RecordAsMuchAsPossible;
        static const char* EchoToConsole;
    }; // RecordModeEnum

    bool hasRecordMode() { return m_recordMode.isJust(); }
    String getRecordMode(const String& defaultValue) { return m_recordMode.isJust() ? m_recordMode.fromJust() : defaultValue; }
    void setRecordMode(const String& value) { m_recordMode = value; }

    bool hasEnableSampling() { return m_enableSampling.isJust(); }
    bool getEnableSampling(bool defaultValue) { return m_enableSampling.isJust() ? m_enableSampling.fromJust() : defaultValue; }
    void setEnableSampling(bool value) { m_enableSampling = value; }

    bool hasEnableSystrace() { return m_enableSystrace.isJust(); }
    bool getEnableSystrace(bool defaultValue) { return m_enableSystrace.isJust() ? m_enableSystrace.fromJust() : defaultValue; }
    void setEnableSystrace(bool value) { m_enableSystrace = value; }

    bool hasEnableArgumentFilter() { return m_enableArgumentFilter.isJust(); }
    bool getEnableArgumentFilter(bool defaultValue) { return m_enableArgumentFilter.isJust() ? m_enableArgumentFilter.fromJust() : defaultValue; }
    void setEnableArgumentFilter(bool value) { m_enableArgumentFilter = value; }

    bool hasIncludedCategories() { return m_includedCategories.isJust(); }
    protocol::Array<String>* getIncludedCategories(protocol::Array<String>* defaultValue) { return m_includedCategories.isJust() ? m_includedCategories.fromJust() : defaultValue; }
    void setIncludedCategories(std::unique_ptr<protocol::Array<String>> value) { m_includedCategories = std::move(value); }

    bool hasExcludedCategories() { return m_excludedCategories.isJust(); }
    protocol::Array<String>* getExcludedCategories(protocol::Array<String>* defaultValue) { return m_excludedCategories.isJust() ? m_excludedCategories.fromJust() : defaultValue; }
    void setExcludedCategories(std::unique_ptr<protocol::Array<String>> value) { m_excludedCategories = std::move(value); }

    bool hasSyntheticDelays() { return m_syntheticDelays.isJust(); }
    protocol::Array<String>* getSyntheticDelays(protocol::Array<String>* defaultValue) { return m_syntheticDelays.isJust() ? m_syntheticDelays.fromJust() : defaultValue; }
    void setSyntheticDelays(std::unique_ptr<protocol::Array<String>> value) { m_syntheticDelays = std::move(value); }

    bool hasMemoryDumpConfig() { return m_memoryDumpConfig.isJust(); }
    protocol::Tracing::MemoryDumpConfig* getMemoryDumpConfig(protocol::Tracing::MemoryDumpConfig* defaultValue) { return m_memoryDumpConfig.isJust() ? m_memoryDumpConfig.fromJust() : defaultValue; }
    void setMemoryDumpConfig(std::unique_ptr<protocol::Tracing::MemoryDumpConfig> value) { m_memoryDumpConfig = std::move(value); }

    std::unique_ptr<protocol::DictionaryValue> serialize() const;
    std::unique_ptr<TraceConfig> clone() const;

    template<int STATE>
    class TraceConfigBuilder {
    public:
        enum {
            NoFieldsSet = 0,
            AllFieldsSet = (0)};


        TraceConfigBuilder<STATE>& setRecordMode(const String& value)
        {
            m_result->setRecordMode(value);
            return *this;
        }

        TraceConfigBuilder<STATE>& setEnableSampling(bool value)
        {
            m_result->setEnableSampling(value);
            return *this;
        }

        TraceConfigBuilder<STATE>& setEnableSystrace(bool value)
        {
            m_result->setEnableSystrace(value);
            return *this;
        }

        TraceConfigBuilder<STATE>& setEnableArgumentFilter(bool value)
        {
            m_result->setEnableArgumentFilter(value);
            return *this;
        }

        TraceConfigBuilder<STATE>& setIncludedCategories(std::unique_ptr<protocol::Array<String>> value)
        {
            m_result->setIncludedCategories(std::move(value));
            return *this;
        }

        TraceConfigBuilder<STATE>& setExcludedCategories(std::unique_ptr<protocol::Array<String>> value)
        {
            m_result->setExcludedCategories(std::move(value));
            return *this;
        }

        TraceConfigBuilder<STATE>& setSyntheticDelays(std::unique_ptr<protocol::Array<String>> value)
        {
            m_result->setSyntheticDelays(std::move(value));
            return *this;
        }

        TraceConfigBuilder<STATE>& setMemoryDumpConfig(std::unique_ptr<protocol::Tracing::MemoryDumpConfig> value)
        {
            m_result->setMemoryDumpConfig(std::move(value));
            return *this;
        }

        std::unique_ptr<TraceConfig> build()
        {
            static_assert(STATE == AllFieldsSet, "state should be AllFieldsSet");
            return std::move(m_result);
        }

    private:
        friend class TraceConfig;
        TraceConfigBuilder() : m_result(new TraceConfig()) { }

        template<int STEP> TraceConfigBuilder<STATE | STEP>& castState()
        {
            return *reinterpret_cast<TraceConfigBuilder<STATE | STEP>*>(this);
        }

        std::unique_ptr<protocol::Tracing::TraceConfig> m_result;
    };

    static TraceConfigBuilder<0> create()
    {
        return TraceConfigBuilder<0>();
    }

private:
    TraceConfig()
    {
    }

    Maybe<String> m_recordMode;
    Maybe<bool> m_enableSampling;
    Maybe<bool> m_enableSystrace;
    Maybe<bool> m_enableArgumentFilter;
    Maybe<protocol::Array<String>> m_includedCategories;
    Maybe<protocol::Array<String>> m_excludedCategories;
    Maybe<protocol::Array<String>> m_syntheticDelays;
    Maybe<protocol::Tracing::MemoryDumpConfig> m_memoryDumpConfig;
};


// ------------- Backend interface.

class CORE_EXPORT Backend {
public:
    virtual ~Backend() { }

    class CORE_EXPORT StartCallback {
    public:
        virtual void sendSuccess() = 0;
        virtual void sendFailure(const DispatchResponse&) = 0;
        virtual ~StartCallback() { }
    };
    virtual void start(Maybe<String> in_categories, Maybe<String> in_options, Maybe<double> in_bufferUsageReportingInterval, Maybe<String> in_transferMode, Maybe<protocol::Tracing::TraceConfig> in_traceConfig, std::unique_ptr<StartCallback> callback) = 0;
    class CORE_EXPORT EndCallback {
    public:
        virtual void sendSuccess() = 0;
        virtual void sendFailure(const DispatchResponse&) = 0;
        virtual ~EndCallback() { }
    };
    virtual void end(std::unique_ptr<EndCallback> callback) = 0;

    virtual DispatchResponse disable()
    {
        return DispatchResponse::OK();
    }
};

// ------------- Frontend interface.

class CORE_EXPORT Frontend {
public:
    explicit Frontend(FrontendChannel* frontendChannel) : m_frontendChannel(frontendChannel) { }
    void dataCollected(std::unique_ptr<protocol::Array<protocol::DictionaryValue>> value);
    void tracingComplete(Maybe<String> stream = Maybe<String>());
    void bufferUsage(Maybe<double> percentFull = Maybe<double>(), Maybe<double> eventCount = Maybe<double>(), Maybe<double> value = Maybe<double>());

    void flush();
private:
    FrontendChannel* m_frontendChannel;
};

// ------------- Dispatcher.

class CORE_EXPORT Dispatcher {
public:
    static void wire(UberDispatcher*, Backend*);

private:
    Dispatcher() { }
};

// ------------- Metainfo.

class CORE_EXPORT Metainfo {
public:
    using BackendClass = Backend;
    using FrontendClass = Frontend;
    using DispatcherClass = Dispatcher;
    static const char domainName[];
    static const char commandPrefix[];
    static const char version[];
};

} // namespace Tracing
} // namespace blink
} // namespace protocol

#endif // !defined(blink_protocol_Tracing_h)
