// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEVICE_SERIAL_SERIAL_MOJOM_SHARED_INTERNAL_H_
#define DEVICE_SERIAL_SERIAL_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace device {
namespace serial {
namespace internal {
class DeviceInfo_Data;
class ConnectionOptions_Data;
class ConnectionInfo_Data;
class HostControlSignals_Data;
class DeviceControlSignals_Data;

struct SendError_Data {
 public:
  static bool const kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct ReceiveError_Data {
 public:
  static bool const kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
      case 7:
      case 8:
      case 9:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct DataBits_Data {
 public:
  static bool const kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct ParityBit_Data {
 public:
  static bool const kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct StopBits_Data {
 public:
  static bool const kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class DeviceInfo_Data {
 public:
  static DeviceInfo_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DeviceInfo_Data))) DeviceInfo_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> path;
  uint16_t vendor_id;
  uint8_t has_vendor_id : 1;
  uint8_t has_product_id : 1;
  uint8_t pad3_[1];
  uint16_t product_id;
  uint8_t pad4_[2];
  mojo::internal::Pointer<mojo::internal::String_Data> display_name;

 private:
  DeviceInfo_Data() : header_({sizeof(*this), 0}) {
  }
  ~DeviceInfo_Data() = delete;
};
static_assert(sizeof(DeviceInfo_Data) == 32,
              "Bad sizeof(DeviceInfo_Data)");
class ConnectionOptions_Data {
 public:
  static ConnectionOptions_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(ConnectionOptions_Data))) ConnectionOptions_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t bitrate;
  int32_t data_bits;
  int32_t parity_bit;
  int32_t stop_bits;
  uint8_t cts_flow_control : 1;
  uint8_t has_cts_flow_control : 1;
  uint8_t padfinal_[7];

 private:
  ConnectionOptions_Data() : header_({sizeof(*this), 0}) {
  }
  ~ConnectionOptions_Data() = delete;
};
static_assert(sizeof(ConnectionOptions_Data) == 32,
              "Bad sizeof(ConnectionOptions_Data)");
class ConnectionInfo_Data {
 public:
  static ConnectionInfo_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(ConnectionInfo_Data))) ConnectionInfo_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t bitrate;
  int32_t data_bits;
  int32_t parity_bit;
  int32_t stop_bits;
  uint8_t cts_flow_control : 1;
  uint8_t padfinal_[7];

 private:
  ConnectionInfo_Data() : header_({sizeof(*this), 0}) {
  }
  ~ConnectionInfo_Data() = delete;
};
static_assert(sizeof(ConnectionInfo_Data) == 32,
              "Bad sizeof(ConnectionInfo_Data)");
class HostControlSignals_Data {
 public:
  static HostControlSignals_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(HostControlSignals_Data))) HostControlSignals_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t dtr : 1;
  uint8_t has_dtr : 1;
  uint8_t rts : 1;
  uint8_t has_rts : 1;
  uint8_t padfinal_[7];

 private:
  HostControlSignals_Data() : header_({sizeof(*this), 0}) {
  }
  ~HostControlSignals_Data() = delete;
};
static_assert(sizeof(HostControlSignals_Data) == 16,
              "Bad sizeof(HostControlSignals_Data)");
class DeviceControlSignals_Data {
 public:
  static DeviceControlSignals_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(DeviceControlSignals_Data))) DeviceControlSignals_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t dcd : 1;
  uint8_t cts : 1;
  uint8_t ri : 1;
  uint8_t dsr : 1;
  uint8_t padfinal_[7];

 private:
  DeviceControlSignals_Data() : header_({sizeof(*this), 0}) {
  }
  ~DeviceControlSignals_Data() = delete;
};
static_assert(sizeof(DeviceControlSignals_Data) == 16,
              "Bad sizeof(DeviceControlSignals_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace serial
}  // namespace device

#endif  // DEVICE_SERIAL_SERIAL_MOJOM_SHARED_INTERNAL_H_