// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MOJO_COMMON_COMMON_CUSTOM_TYPES_MOJOM_BLINK_H_
#define MOJO_COMMON_COMMON_CUSTOM_TYPES_MOJOM_BLINK_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "mojo/common/common_custom_types.mojom-shared.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "mojo/public/cpp/bindings/wtf_array.h"
#include "mojo/public/cpp/bindings/wtf_map.h"
#include "third_party/WebKit/Source/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/wtf/Optional.h"
#include "third_party/WebKit/Source/wtf/text/WTFString.h"
#include "third_party/WebKit/Source/wtf/text/WTFString.h"


namespace mojo {
namespace common {
namespace mojom {
namespace blink {
using FilePath = mojo::NativeStruct;
using FilePathPtr = mojo::NativeStructPtr;

using ListValue = mojo::NativeStruct;
using ListValuePtr = mojo::NativeStructPtr;

using DictionaryValue = mojo::NativeStruct;
using DictionaryValuePtr = mojo::NativeStructPtr;

using Time = mojo::NativeStruct;
using TimePtr = mojo::NativeStructPtr;

class TimeDelta;
using TimeDeltaPtr = mojo::InlinedStructPtr<TimeDelta>;

using TimeTicks = mojo::NativeStruct;
using TimeTicksPtr = mojo::NativeStructPtr;

class String16;
using String16Ptr = mojo::StructPtr<String16>;

class UnguessableToken;
using UnguessableTokenPtr = mojo::InlinedStructPtr<UnguessableToken>;

class Version;
using VersionPtr = mojo::StructPtr<Version>;







class  TimeDelta {
 public:
  using DataView = TimeDeltaDataView;
  using Data_ = internal::TimeDelta_Data;

  static TimeDeltaPtr New();

  template <typename U>
  static TimeDeltaPtr From(const U& u) {
    return mojo::TypeConverter<TimeDeltaPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, TimeDelta>::Convert(*this);
  }

  TimeDelta();
  ~TimeDelta();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = TimeDeltaPtr>
  TimeDeltaPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, TimeDelta>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static mojo::WTFArray<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        TimeDelta::DataView, mojo::WTFArray<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::WTFArray<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        TimeDelta::DataView, mojo::WTFArray<uint8_t>>(
            input, output);
  }

  int64_t microseconds;
};





class  UnguessableToken {
 public:
  using DataView = UnguessableTokenDataView;
  using Data_ = internal::UnguessableToken_Data;

  static UnguessableTokenPtr New();

  template <typename U>
  static UnguessableTokenPtr From(const U& u) {
    return mojo::TypeConverter<UnguessableTokenPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, UnguessableToken>::Convert(*this);
  }

  UnguessableToken();
  ~UnguessableToken();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = UnguessableTokenPtr>
  UnguessableTokenPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, UnguessableToken>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static mojo::WTFArray<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        UnguessableToken::DataView, mojo::WTFArray<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::WTFArray<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        UnguessableToken::DataView, mojo::WTFArray<uint8_t>>(
            input, output);
  }

  uint64_t high;
  uint64_t low;
};












class  String16 {
 public:
  using DataView = String16DataView;
  using Data_ = internal::String16_Data;

  static String16Ptr New();

  template <typename U>
  static String16Ptr From(const U& u) {
    return mojo::TypeConverter<String16Ptr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, String16>::Convert(*this);
  }

  String16();
  ~String16();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = String16Ptr>
  String16Ptr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, String16>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::WTFArray<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        String16::DataView, mojo::WTFArray<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::WTFArray<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        String16::DataView, mojo::WTFArray<uint8_t>>(
            input, output);
  }

  WTF::Vector<uint16_t> data;
};




class  Version {
 public:
  using DataView = VersionDataView;
  using Data_ = internal::Version_Data;

  static VersionPtr New();

  template <typename U>
  static VersionPtr From(const U& u) {
    return mojo::TypeConverter<VersionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Version>::Convert(*this);
  }

  Version();
  ~Version();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = VersionPtr>
  VersionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Version>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::WTFArray<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        Version::DataView, mojo::WTFArray<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::WTFArray<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        Version::DataView, mojo::WTFArray<uint8_t>>(
            input, output);
  }

  WTF::Vector<uint32_t> components;
};


template <typename StructPtrType>
TimeDeltaPtr TimeDelta::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->microseconds = mojo::internal::Clone(microseconds);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, TimeDelta>::value>::type*>
bool TimeDelta::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->microseconds, other.microseconds))
    return false;
  return true;
}
template <typename StructPtrType>
String16Ptr String16::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->data = mojo::internal::Clone(data);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, String16>::value>::type*>
bool String16::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->data, other.data))
    return false;
  return true;
}
template <typename StructPtrType>
UnguessableTokenPtr UnguessableToken::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->high = mojo::internal::Clone(high);
  rv->low = mojo::internal::Clone(low);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, UnguessableToken>::value>::type*>
bool UnguessableToken::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->high, other.high))
    return false;
  if (!mojo::internal::Equals(this->low, other.low))
    return false;
  return true;
}
template <typename StructPtrType>
VersionPtr Version::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->components = mojo::internal::Clone(components);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Version>::value>::type*>
bool Version::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->components, other.components))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace common
}  // namespace mojo

namespace mojo {


template <>
struct  StructTraits<::mojo::common::mojom::blink::TimeDelta::DataView,
                                         ::mojo::common::mojom::blink::TimeDeltaPtr> {
  static bool IsNull(const ::mojo::common::mojom::blink::TimeDeltaPtr& input) { return !input; }
  static void SetToNull(::mojo::common::mojom::blink::TimeDeltaPtr* output) { output->reset(); }

  static decltype(::mojo::common::mojom::blink::TimeDelta::microseconds) microseconds(
      const ::mojo::common::mojom::blink::TimeDeltaPtr& input) {
    return input->microseconds;
  }

  static bool Read(::mojo::common::mojom::blink::TimeDelta::DataView input, ::mojo::common::mojom::blink::TimeDeltaPtr* output);
};


template <>
struct  StructTraits<::mojo::common::mojom::blink::String16::DataView,
                                         ::mojo::common::mojom::blink::String16Ptr> {
  static bool IsNull(const ::mojo::common::mojom::blink::String16Ptr& input) { return !input; }
  static void SetToNull(::mojo::common::mojom::blink::String16Ptr* output) { output->reset(); }

  static const decltype(::mojo::common::mojom::blink::String16::data)& data(
      const ::mojo::common::mojom::blink::String16Ptr& input) {
    return input->data;
  }

  static bool Read(::mojo::common::mojom::blink::String16::DataView input, ::mojo::common::mojom::blink::String16Ptr* output);
};


template <>
struct  StructTraits<::mojo::common::mojom::blink::UnguessableToken::DataView,
                                         ::mojo::common::mojom::blink::UnguessableTokenPtr> {
  static bool IsNull(const ::mojo::common::mojom::blink::UnguessableTokenPtr& input) { return !input; }
  static void SetToNull(::mojo::common::mojom::blink::UnguessableTokenPtr* output) { output->reset(); }

  static decltype(::mojo::common::mojom::blink::UnguessableToken::high) high(
      const ::mojo::common::mojom::blink::UnguessableTokenPtr& input) {
    return input->high;
  }

  static decltype(::mojo::common::mojom::blink::UnguessableToken::low) low(
      const ::mojo::common::mojom::blink::UnguessableTokenPtr& input) {
    return input->low;
  }

  static bool Read(::mojo::common::mojom::blink::UnguessableToken::DataView input, ::mojo::common::mojom::blink::UnguessableTokenPtr* output);
};


template <>
struct  StructTraits<::mojo::common::mojom::blink::Version::DataView,
                                         ::mojo::common::mojom::blink::VersionPtr> {
  static bool IsNull(const ::mojo::common::mojom::blink::VersionPtr& input) { return !input; }
  static void SetToNull(::mojo::common::mojom::blink::VersionPtr* output) { output->reset(); }

  static const decltype(::mojo::common::mojom::blink::Version::components)& components(
      const ::mojo::common::mojom::blink::VersionPtr& input) {
    return input->components;
  }

  static bool Read(::mojo::common::mojom::blink::Version::DataView input, ::mojo::common::mojom::blink::VersionPtr* output);
};

}  // namespace mojo

#endif  // MOJO_COMMON_COMMON_CUSTOM_TYPES_MOJOM_BLINK_H_