// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_SHARED_H_
#define SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/ui/public/interfaces/animations.mojom-shared-internal.h"
#include "ui/gfx/mojo/transform.mojom-shared.h"


namespace ui {
namespace mojom {
class AnimationValueDataView;

class AnimationElementDataView;

class AnimationSequenceDataView;

class AnimationGroupDataView;



}  // namespace mojom
}  // namespace ui

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::ui::mojom::AnimationValueDataView> {
  using Data = ::ui::mojom::internal::AnimationValue_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::AnimationElementDataView> {
  using Data = ::ui::mojom::internal::AnimationElement_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::AnimationSequenceDataView> {
  using Data = ::ui::mojom::internal::AnimationSequence_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::ui::mojom::AnimationGroupDataView> {
  using Data = ::ui::mojom::internal::AnimationGroup_Data;
  using DataAsArrayElement = Pointer<Data>;
  static const MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace ui {
namespace mojom {

enum class AnimationTweenType : int32_t {
  LINEAR,
  EASE_IN,
  EASE_OUT,
  EASE_IN_OUT,
};

inline std::ostream& operator<<(std::ostream& os, AnimationTweenType value) {
  switch(value) {
    case AnimationTweenType::LINEAR:
      return os << "AnimationTweenType::LINEAR";
    case AnimationTweenType::EASE_IN:
      return os << "AnimationTweenType::EASE_IN";
    case AnimationTweenType::EASE_OUT:
      return os << "AnimationTweenType::EASE_OUT";
    case AnimationTweenType::EASE_IN_OUT:
      return os << "AnimationTweenType::EASE_IN_OUT";
    default:
      return os << "Unknown AnimationTweenType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AnimationTweenType value) {
  return internal::AnimationTweenType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class AnimationProperty : int32_t {
  NONE,
  OPACITY,
  TRANSFORM,
};

inline std::ostream& operator<<(std::ostream& os, AnimationProperty value) {
  switch(value) {
    case AnimationProperty::NONE:
      return os << "AnimationProperty::NONE";
    case AnimationProperty::OPACITY:
      return os << "AnimationProperty::OPACITY";
    case AnimationProperty::TRANSFORM:
      return os << "AnimationProperty::TRANSFORM";
    default:
      return os << "Unknown AnimationProperty value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AnimationProperty value) {
  return internal::AnimationProperty_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class AnimationValueDataView {
 public:
  AnimationValueDataView() {}

  AnimationValueDataView(
      internal::AnimationValue_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  float float_value() const {
    return data_->float_value;
  }
  inline void GetTransformDataView(
      ::gfx::mojom::TransformDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransform(UserType* output) {
    auto* pointer = data_->transform.Get();
    return mojo::internal::Deserialize<::gfx::mojom::TransformDataView>(
        pointer, output, context_);
  }
 private:
  internal::AnimationValue_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AnimationElementDataView {
 public:
  AnimationElementDataView() {}

  AnimationElementDataView(
      internal::AnimationElement_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProperty(UserType* output) const {
    auto data_value = data_->property;
    return mojo::internal::Deserialize<::ui::mojom::AnimationProperty>(
        data_value, output);
  }

  AnimationProperty property() const {
    return static_cast<AnimationProperty>(data_->property);
  }
  int64_t duration() const {
    return data_->duration;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTweenType(UserType* output) const {
    auto data_value = data_->tween_type;
    return mojo::internal::Deserialize<::ui::mojom::AnimationTweenType>(
        data_value, output);
  }

  AnimationTweenType tween_type() const {
    return static_cast<AnimationTweenType>(data_->tween_type);
  }
  inline void GetStartValueDataView(
      AnimationValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartValue(UserType* output) {
    auto* pointer = data_->start_value.Get();
    return mojo::internal::Deserialize<::ui::mojom::AnimationValueDataView>(
        pointer, output, context_);
  }
  inline void GetTargetValueDataView(
      AnimationValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTargetValue(UserType* output) {
    auto* pointer = data_->target_value.Get();
    return mojo::internal::Deserialize<::ui::mojom::AnimationValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::AnimationElement_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AnimationSequenceDataView {
 public:
  AnimationSequenceDataView() {}

  AnimationSequenceDataView(
      internal::AnimationSequence_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t cycle_count() const {
    return data_->cycle_count;
  }
  inline void GetElementsDataView(
      mojo::ArrayDataView<AnimationElementDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadElements(UserType* output) {
    auto* pointer = data_->elements.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::ui::mojom::AnimationElementDataView>>(
        pointer, output, context_);
  }
 private:
  internal::AnimationSequence_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AnimationGroupDataView {
 public:
  AnimationGroupDataView() {}

  AnimationGroupDataView(
      internal::AnimationGroup_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t window_id() const {
    return data_->window_id;
  }
  inline void GetSequencesDataView(
      mojo::ArrayDataView<AnimationSequenceDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSequences(UserType* output) {
    auto* pointer = data_->sequences.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::ui::mojom::AnimationSequenceDataView>>(
        pointer, output, context_);
  }
 private:
  internal::AnimationGroup_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace ui

namespace std {

template <>
struct hash<::ui::mojom::AnimationTweenType>
    : public mojo::internal::EnumHashImpl<::ui::mojom::AnimationTweenType> {};

template <>
struct hash<::ui::mojom::AnimationProperty>
    : public mojo::internal::EnumHashImpl<::ui::mojom::AnimationProperty> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::ui::mojom::AnimationTweenType, ::ui::mojom::AnimationTweenType> {
  static ::ui::mojom::AnimationTweenType ToMojom(::ui::mojom::AnimationTweenType input) { return input; }
  static bool FromMojom(::ui::mojom::AnimationTweenType input, ::ui::mojom::AnimationTweenType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationTweenType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::AnimationTweenType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::AnimationTweenType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::ui::mojom::AnimationProperty, ::ui::mojom::AnimationProperty> {
  static ::ui::mojom::AnimationProperty ToMojom(::ui::mojom::AnimationProperty input) { return input; }
  static bool FromMojom(::ui::mojom::AnimationProperty input, ::ui::mojom::AnimationProperty* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationProperty, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::ui::mojom::AnimationProperty, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::ui::mojom::AnimationProperty>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationValueDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::AnimationValueDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::AnimationValue_Data);
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
        in_transform, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::AnimationValue_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::AnimationValue_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->float_value = CallWithContext(Traits::float_value, input, custom_context);
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    typename decltype(result->transform)::BaseType* transform_ptr;
    mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
        in_transform, buffer, &transform_ptr, context);
    result->transform.Set(transform_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::AnimationValue_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::AnimationValueDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationElementDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::AnimationElementDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::AnimationElement_Data);
    decltype(CallWithContext(Traits::start_value, input, custom_context)) in_start_value = CallWithContext(Traits::start_value, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::ui::mojom::AnimationValueDataView>(
        in_start_value, context);
    decltype(CallWithContext(Traits::target_value, input, custom_context)) in_target_value = CallWithContext(Traits::target_value, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::ui::mojom::AnimationValueDataView>(
        in_target_value, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::AnimationElement_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::AnimationElement_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    mojo::internal::Serialize<::ui::mojom::AnimationProperty>(
        CallWithContext(Traits::property, input, custom_context), &result->property);
    result->duration = CallWithContext(Traits::duration, input, custom_context);
    mojo::internal::Serialize<::ui::mojom::AnimationTweenType>(
        CallWithContext(Traits::tween_type, input, custom_context), &result->tween_type);
    decltype(CallWithContext(Traits::start_value, input, custom_context)) in_start_value = CallWithContext(Traits::start_value, input, custom_context);
    typename decltype(result->start_value)::BaseType* start_value_ptr;
    mojo::internal::Serialize<::ui::mojom::AnimationValueDataView>(
        in_start_value, buffer, &start_value_ptr, context);
    result->start_value.Set(start_value_ptr);
    decltype(CallWithContext(Traits::target_value, input, custom_context)) in_target_value = CallWithContext(Traits::target_value, input, custom_context);
    typename decltype(result->target_value)::BaseType* target_value_ptr;
    mojo::internal::Serialize<::ui::mojom::AnimationValueDataView>(
        in_target_value, buffer, &target_value_ptr, context);
    result->target_value.Set(target_value_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::AnimationElement_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::AnimationElementDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationSequenceDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::AnimationSequenceDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::AnimationSequence_Data);
    decltype(CallWithContext(Traits::elements, input, custom_context)) in_elements = CallWithContext(Traits::elements, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::ui::mojom::AnimationElementDataView>>(
        in_elements, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::AnimationSequence_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::AnimationSequence_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->cycle_count = CallWithContext(Traits::cycle_count, input, custom_context);
    decltype(CallWithContext(Traits::elements, input, custom_context)) in_elements = CallWithContext(Traits::elements, input, custom_context);
    typename decltype(result->elements)::BaseType* elements_ptr;
    const mojo::internal::ContainerValidateParams elements_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::ui::mojom::AnimationElementDataView>>(
        in_elements, buffer, &elements_ptr, &elements_validate_params,
        context);
    result->elements.Set(elements_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->elements.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null elements in AnimationSequence struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::AnimationSequence_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::AnimationSequenceDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::ui::mojom::AnimationGroupDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::ui::mojom::AnimationGroupDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input))
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::ui::mojom::internal::AnimationGroup_Data);
    decltype(CallWithContext(Traits::sequences, input, custom_context)) in_sequences = CallWithContext(Traits::sequences, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::ui::mojom::AnimationSequenceDataView>>(
        in_sequences, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::ui::mojom::internal::AnimationGroup_Data** output,
                        SerializationContext* context) {
    if (CallIsNullIfExists<Traits>(input)) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::ui::mojom::internal::AnimationGroup_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->window_id = CallWithContext(Traits::window_id, input, custom_context);
    decltype(CallWithContext(Traits::sequences, input, custom_context)) in_sequences = CallWithContext(Traits::sequences, input, custom_context);
    typename decltype(result->sequences)::BaseType* sequences_ptr;
    const mojo::internal::ContainerValidateParams sequences_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::ui::mojom::AnimationSequenceDataView>>(
        in_sequences, buffer, &sequences_ptr, &sequences_validate_params,
        context);
    result->sequences.Set(sequences_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->sequences.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null sequences in AnimationGroup struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::ui::mojom::internal::AnimationGroup_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::ui::mojom::AnimationGroupDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace ui {
namespace mojom {

inline void AnimationValueDataView::GetTransformDataView(
    ::gfx::mojom::TransformDataView* output) {
  auto pointer = data_->transform.Get();
  *output = ::gfx::mojom::TransformDataView(pointer, context_);
}


inline void AnimationElementDataView::GetStartValueDataView(
    AnimationValueDataView* output) {
  auto pointer = data_->start_value.Get();
  *output = AnimationValueDataView(pointer, context_);
}
inline void AnimationElementDataView::GetTargetValueDataView(
    AnimationValueDataView* output) {
  auto pointer = data_->target_value.Get();
  *output = AnimationValueDataView(pointer, context_);
}


inline void AnimationSequenceDataView::GetElementsDataView(
    mojo::ArrayDataView<AnimationElementDataView>* output) {
  auto pointer = data_->elements.Get();
  *output = mojo::ArrayDataView<AnimationElementDataView>(pointer, context_);
}


inline void AnimationGroupDataView::GetSequencesDataView(
    mojo::ArrayDataView<AnimationSequenceDataView>* output) {
  auto pointer = data_->sequences.Get();
  *output = mojo::ArrayDataView<AnimationSequenceDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace ui

#endif  // SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_SHARED_H_
