// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_H_
#define SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/public/interfaces/animations.mojom-shared.h"
#include "ui/gfx/mojo/transform.mojom.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "gpu/config/dx_diag_node.h"
#include "base/values.h"
#include "cc/output/compositor_frame.h"
#include "third_party/skia/include/core/SkImageFilter.h"
#include "gpu/command_buffer/common/sync_token.h"
#include "cc/output/filter_operation.h"
#include "ui/display/display.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "ui/gfx/icc_profile.h"
#include "ui/gfx/geometry/insets.h"
#include "base/version.h"
#include "gpu/config/gpu_info.h"
#include "base/time/time.h"
#include "ui/gfx/gpu_memory_buffer.h"
#include "cc/output/filter_operations.h"
#include "base/strings/string16.h"
#include "ui/gfx/selection_bound.h"
#include "base/unguessable_token.h"
#include "ui/gfx/geometry/size.h"
#include "ui/gfx/geometry/point.h"
#include "ui/events/latency_info.h"
#include "cc/output/begin_frame_args.h"
#include "cc/surfaces/surface_sequence.h"
#include "cc/output/compositor_frame_metadata.h"
#include "ui/events/event.h"
#include "cc/quads/draw_quad.h"
#include "ui/gfx/native_pixmap_handle.h"
#include "ui/gfx/geometry/rect_f.h"
#include "cc/quads/render_pass.h"
#include "gpu/command_buffer/common/mailbox_holder.h"
#include "cc/resources/returned_resource.h"
#include "base/files/file_path.h"
#include "cc/surfaces/surface_id.h"
#include "ui/gfx/geometry/point_f.h"
#include "gpu/command_buffer/common/mailbox.h"
#include "cc/input/selection.h"
#include "third_party/skia/include/core/SkBitmap.h"
#include "ui/gfx/buffer_types.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "cc/surfaces/frame_sink_id.h"
#include "ui/gfx/native_widget_types.h"
#include "cc/quads/shared_quad_state.h"
#include "cc/resources/transferable_resource.h"
#include "ui/gfx/transform.h"
#include "cc/surfaces/local_frame_id.h"
#include "ui/gfx/geometry/rect.h"
#include "gpu/command_buffer/common/capabilities.h"
#include "gpu/ipc/common/surface_handle.h"
#include "ui/gfx/geometry/vector2d.h"
#include "cc/quads/render_pass_id.h"


namespace ui {
namespace mojom {
class AnimationValue;
using AnimationValuePtr = mojo::StructPtr<AnimationValue>;

class AnimationElement;
using AnimationElementPtr = mojo::StructPtr<AnimationElement>;

class AnimationSequence;
using AnimationSequencePtr = mojo::StructPtr<AnimationSequence>;

class AnimationGroup;
using AnimationGroupPtr = mojo::StructPtr<AnimationGroup>;









class  AnimationValue {
 public:
  using DataView = AnimationValueDataView;
  using Data_ = internal::AnimationValue_Data;

  static AnimationValuePtr New();

  template <typename U>
  static AnimationValuePtr From(const U& u) {
    return mojo::TypeConverter<AnimationValuePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AnimationValue>::Convert(*this);
  }

  AnimationValue();
  ~AnimationValue();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AnimationValuePtr>
  AnimationValuePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AnimationValue>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        AnimationValue::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        AnimationValue::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  float float_value;
  base::Optional<gfx::Transform> transform;
};



class  AnimationElement {
 public:
  using DataView = AnimationElementDataView;
  using Data_ = internal::AnimationElement_Data;

  static AnimationElementPtr New();

  template <typename U>
  static AnimationElementPtr From(const U& u) {
    return mojo::TypeConverter<AnimationElementPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AnimationElement>::Convert(*this);
  }

  AnimationElement();
  ~AnimationElement();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AnimationElementPtr>
  AnimationElementPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AnimationElement>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        AnimationElement::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        AnimationElement::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  AnimationProperty property;
  int64_t duration;
  AnimationTweenType tween_type;
  AnimationValuePtr start_value;
  AnimationValuePtr target_value;
 private:
  DISALLOW_COPY_AND_ASSIGN(AnimationElement);
};



class  AnimationSequence {
 public:
  using DataView = AnimationSequenceDataView;
  using Data_ = internal::AnimationSequence_Data;

  static AnimationSequencePtr New();

  template <typename U>
  static AnimationSequencePtr From(const U& u) {
    return mojo::TypeConverter<AnimationSequencePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AnimationSequence>::Convert(*this);
  }

  AnimationSequence();
  ~AnimationSequence();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AnimationSequencePtr>
  AnimationSequencePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AnimationSequence>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        AnimationSequence::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        AnimationSequence::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  uint32_t cycle_count;
  std::vector<AnimationElementPtr> elements;
 private:
  DISALLOW_COPY_AND_ASSIGN(AnimationSequence);
};



class  AnimationGroup {
 public:
  using DataView = AnimationGroupDataView;
  using Data_ = internal::AnimationGroup_Data;

  static AnimationGroupPtr New();

  template <typename U>
  static AnimationGroupPtr From(const U& u) {
    return mojo::TypeConverter<AnimationGroupPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, AnimationGroup>::Convert(*this);
  }

  AnimationGroup();
  ~AnimationGroup();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = AnimationGroupPtr>
  AnimationGroupPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, AnimationGroup>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static mojo::Array<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        AnimationGroup::DataView, mojo::Array<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const mojo::Array<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        AnimationGroup::DataView, mojo::Array<uint8_t>>(
            input, output);
  }

  uint32_t window_id;
  std::vector<AnimationSequencePtr> sequences;
 private:
  DISALLOW_COPY_AND_ASSIGN(AnimationGroup);
};


template <typename StructPtrType>
AnimationValuePtr AnimationValue::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->float_value = mojo::internal::Clone(float_value);
  rv->transform = mojo::internal::Clone(transform);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AnimationValue>::value>::type*>
bool AnimationValue::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->float_value, other.float_value))
    return false;
  if (!mojo::internal::Equals(this->transform, other.transform))
    return false;
  return true;
}
template <typename StructPtrType>
AnimationElementPtr AnimationElement::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->property = mojo::internal::Clone(property);
  rv->duration = mojo::internal::Clone(duration);
  rv->tween_type = mojo::internal::Clone(tween_type);
  rv->start_value = mojo::internal::Clone(start_value);
  rv->target_value = mojo::internal::Clone(target_value);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AnimationElement>::value>::type*>
bool AnimationElement::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->property, other.property))
    return false;
  if (!mojo::internal::Equals(this->duration, other.duration))
    return false;
  if (!mojo::internal::Equals(this->tween_type, other.tween_type))
    return false;
  if (!mojo::internal::Equals(this->start_value, other.start_value))
    return false;
  if (!mojo::internal::Equals(this->target_value, other.target_value))
    return false;
  return true;
}
template <typename StructPtrType>
AnimationSequencePtr AnimationSequence::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->cycle_count = mojo::internal::Clone(cycle_count);
  rv->elements = mojo::internal::Clone(elements);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AnimationSequence>::value>::type*>
bool AnimationSequence::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->cycle_count, other.cycle_count))
    return false;
  if (!mojo::internal::Equals(this->elements, other.elements))
    return false;
  return true;
}
template <typename StructPtrType>
AnimationGroupPtr AnimationGroup::Clone() const {
  // Use StructPtrType to prevent the compiler from trying to compile this
  // without being asked.
  StructPtrType rv(New());
  rv->window_id = mojo::internal::Clone(window_id);
  rv->sequences = mojo::internal::Clone(sequences);
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, AnimationGroup>::value>::type*>
bool AnimationGroup::Equals(const T& other) const {
  if (!mojo::internal::Equals(this->window_id, other.window_id))
    return false;
  if (!mojo::internal::Equals(this->sequences, other.sequences))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace ui

namespace mojo {


template <>
struct  StructTraits<::ui::mojom::AnimationValue::DataView,
                                         ::ui::mojom::AnimationValuePtr> {
  static bool IsNull(const ::ui::mojom::AnimationValuePtr& input) { return !input; }
  static void SetToNull(::ui::mojom::AnimationValuePtr* output) { output->reset(); }

  static decltype(::ui::mojom::AnimationValue::float_value) float_value(
      const ::ui::mojom::AnimationValuePtr& input) {
    return input->float_value;
  }

  static const decltype(::ui::mojom::AnimationValue::transform)& transform(
      const ::ui::mojom::AnimationValuePtr& input) {
    return input->transform;
  }

  static bool Read(::ui::mojom::AnimationValue::DataView input, ::ui::mojom::AnimationValuePtr* output);
};


template <>
struct  StructTraits<::ui::mojom::AnimationElement::DataView,
                                         ::ui::mojom::AnimationElementPtr> {
  static bool IsNull(const ::ui::mojom::AnimationElementPtr& input) { return !input; }
  static void SetToNull(::ui::mojom::AnimationElementPtr* output) { output->reset(); }

  static decltype(::ui::mojom::AnimationElement::property) property(
      const ::ui::mojom::AnimationElementPtr& input) {
    return input->property;
  }

  static decltype(::ui::mojom::AnimationElement::duration) duration(
      const ::ui::mojom::AnimationElementPtr& input) {
    return input->duration;
  }

  static decltype(::ui::mojom::AnimationElement::tween_type) tween_type(
      const ::ui::mojom::AnimationElementPtr& input) {
    return input->tween_type;
  }

  static const decltype(::ui::mojom::AnimationElement::start_value)& start_value(
      const ::ui::mojom::AnimationElementPtr& input) {
    return input->start_value;
  }

  static const decltype(::ui::mojom::AnimationElement::target_value)& target_value(
      const ::ui::mojom::AnimationElementPtr& input) {
    return input->target_value;
  }

  static bool Read(::ui::mojom::AnimationElement::DataView input, ::ui::mojom::AnimationElementPtr* output);
};


template <>
struct  StructTraits<::ui::mojom::AnimationSequence::DataView,
                                         ::ui::mojom::AnimationSequencePtr> {
  static bool IsNull(const ::ui::mojom::AnimationSequencePtr& input) { return !input; }
  static void SetToNull(::ui::mojom::AnimationSequencePtr* output) { output->reset(); }

  static decltype(::ui::mojom::AnimationSequence::cycle_count) cycle_count(
      const ::ui::mojom::AnimationSequencePtr& input) {
    return input->cycle_count;
  }

  static const decltype(::ui::mojom::AnimationSequence::elements)& elements(
      const ::ui::mojom::AnimationSequencePtr& input) {
    return input->elements;
  }

  static bool Read(::ui::mojom::AnimationSequence::DataView input, ::ui::mojom::AnimationSequencePtr* output);
};


template <>
struct  StructTraits<::ui::mojom::AnimationGroup::DataView,
                                         ::ui::mojom::AnimationGroupPtr> {
  static bool IsNull(const ::ui::mojom::AnimationGroupPtr& input) { return !input; }
  static void SetToNull(::ui::mojom::AnimationGroupPtr* output) { output->reset(); }

  static decltype(::ui::mojom::AnimationGroup::window_id) window_id(
      const ::ui::mojom::AnimationGroupPtr& input) {
    return input->window_id;
  }

  static const decltype(::ui::mojom::AnimationGroup::sequences)& sequences(
      const ::ui::mojom::AnimationGroupPtr& input) {
    return input->sequences;
  }

  static bool Read(::ui::mojom::AnimationGroup::DataView input, ::ui::mojom::AnimationGroupPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_UI_PUBLIC_INTERFACES_ANIMATIONS_MOJOM_H_