// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_DEVICE_MOJOM_H_
#define SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_DEVICE_MOJOM_H_

#include <stdint.h>

#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/lib/union_accessor.h"
#include "mojo/public/cpp/bindings/map.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/no_interface.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/video_capture/public/interfaces/device.mojom-shared.h"
#include "media/capture/mojo/video_capture_types.mojom.h"
#include "services/video_capture/public/interfaces/capture_settings.mojom.h"
#include "services/video_capture/public/interfaces/receiver.mojom.h"
#include "mojo/public/cpp/bindings/array.h"
#include "mojo/public/cpp/bindings/string.h"
#include "base/values.h"
#include "media/base/subsample_entry.h"
#include "media/base/decryptor.h"
#include "ui/gfx/geometry/vector2d_f.h"
#include "ui/gfx/geometry/insets.h"
#include "base/version.h"
#include "media/capture/video_capture_types.h"
#include "base/time/time.h"
#include "base/strings/string16.h"
#include "media/base/sample_format.h"
#include "services/service_manager/public/cpp/identity.h"
#include "media/base/demuxer_stream.h"
#include "ui/gfx/geometry/size.h"
#include "media/base/eme_constants.h"
#include "media/base/video_codecs.h"
#include "ui/gfx/geometry/point.h"
#include "ui/gfx/geometry/rect.h"
#include "media/base/cdm_key_information.h"
#include "media/base/media_keys.h"
#include "media/base/audio_codecs.h"
#include "media/base/audio_parameters.h"
#include "ui/gfx/geometry/rect_f.h"
#include "media/base/cdm_promise.h"
#include "base/files/file_path.h"
#include "media/base/buffering_state.h"
#include "ui/gfx/geometry/point_f.h"
#include "media/base/pipeline_status.h"
#include "base/unguessable_token.h"
#include "services/service_manager/public/cpp/service_info.h"
#include "ui/gfx/geometry/safe_integer_conversions.h"
#include "url/gurl.h"
#include "media/base/decode_status.h"
#include "media/capture/video/video_capture_device_descriptor.h"
#include "media/base/encryption_scheme.h"
#include "media/base/channel_layout.h"
#include "ui/gfx/geometry/vector2d.h"
#include "media/base/video_types.h"
#include "services/video_capture/public/cpp/capture_settings.h"
#include "services/service_manager/public/cpp/interface_provider_spec.h"


namespace video_capture {
namespace mojom {
class Device;
using DevicePtr = mojo::InterfacePtr<Device>;
using DevicePtrInfo = mojo::InterfacePtrInfo<Device>;
using ThreadSafeDevicePtr =
    mojo::ThreadSafeInterfacePtr<Device>;
using DeviceRequest = mojo::InterfaceRequest<Device>;
using DeviceAssociatedPtr =
    mojo::AssociatedInterfacePtr<Device>;
using ThreadSafeDeviceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Device>;
using DeviceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Device>;
using DeviceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Device>;


class DeviceProxy;

template <typename ImplRefTraits>
class DeviceStub;

class DeviceRequestValidator;

class  Device
    : public DeviceInterfaceBase {
 public:
  static const char Name_[];
  static const uint32_t Version_ = 0;
  static const bool PassesAssociatedKinds_ = false;
  static const bool HasSyncMethods_ = false;

  using Proxy_ = DeviceProxy;

  template <typename ImplRefTraits>
  using Stub_ = DeviceStub<ImplRefTraits>;

  using RequestValidator_ = DeviceRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  virtual ~Device() {}

  virtual void Start(const video_capture::CaptureSettings& requested_settings, ::video_capture::mojom::ReceiverPtr receiver) = 0;
};

class  DeviceProxy
    : public Device {
 public:
  explicit DeviceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Start(const video_capture::CaptureSettings& requested_settings, ::video_capture::mojom::ReceiverPtr receiver) override;

  void set_group_controller(
        scoped_refptr<mojo::AssociatedGroupController> group_controller) {
    group_controller_ = std::move(group_controller);
  }
 private:
  mojo::MessageReceiverWithResponder* receiver_;
  scoped_refptr<mojo::AssociatedGroupController> group_controller_;
};
class  DeviceStubDispatch {
 public:
  static bool Accept(Device* impl,
                     mojo::internal::SerializationContext* context,
                     mojo::Message* message);
  static bool AcceptWithResponder(Device* impl,
                                  mojo::internal::SerializationContext* context,
                                  mojo::Message* message,
                                  mojo::MessageReceiverWithStatus* responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Device>>
class DeviceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  DeviceStub() {}
  ~DeviceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  mojo::internal::SerializationContext* serialization_context() {
    return &serialization_context_;
  }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DeviceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      mojo::MessageReceiverWithStatus* responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return DeviceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), &serialization_context_, message,
        responder);
  }

 private:
  ImplPointerType sink_;
  mojo::internal::SerializationContext serialization_context_;
};
class  DeviceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace video_capture

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_VIDEO_CAPTURE_PUBLIC_INTERFACES_DEVICE_MOJOM_H_